with device_register_types; use device_register_types;
with system; use system;
with Ada.Real_Time; use Ada.Real_Time;
with air_flow_sensor_rtatt; use air_flow_sensor_rtatt;
with operator_console; use operator_console;
with data_logger; use data_logger;
package body air_flow_sensor is

  air_flow_reading : boolean;
  -- define control and status register
  -- for the flow switch
  --Control_reg_addr : constant Address := 16#AA120#
  afcsr : device_register_types.csr;
  --for afcsr'address use Control_reg_addr;

  --procedure PERIODIC_CODE is separate;
  --procedure initialise is separate;

  task THREAD is
    PRAGMA PRIORITY(air_flow_sensor_rtatt.
                    initial_thread_priority);
  end thread;

--separate(air_flow_sensor)
procedure initialise is
begin
  -- enable device
  afcsr.device := D_enabled;
end initialise;


--separate(air_flow_sensor)
procedure PERIODIC_CODE is
begin
  -- read device register for flow indication 
  -- (operation bit set to 1);
  air_flow_reading := afcsr.operation = set;
  if not air_flow_reading then
    operator_console.alarm(NO_AIR_FLOW);
    data_logger.air_flow_log(no_air_flow);
  else
    data_logger.air_flow_log(air_flow);
  end if;
end PERIODIC_CODE;

  task body THREAD is
    T: TIME;
    PERIOD : Time_Span := air_flow_sensor_rtatt.period;
  begin
    T:= clock;
    initialise;
    loop
      PERIODIC_CODE;
      T := T + PERIOD;
      delay until(T);
    end loop;
  end thread;

end air_flow_sensor;
