with Ada.Real_Time; use Ada.Real_Time;
with system; use system;
with ch4_sensor_rtatt; use ch4_sensor_rtatt;
with device_register_types; use device_register_types;
with operator_console; use operator_console;
with motor; use motor;
with data_logger; use data_logger; 
with ch4_status; use ch4_status;
package body CH4_sensor is

  CH4_present : ch4_reading;
  -- define control and status register
  -- for the CH4 ADC
  --Control_reg_addr : constant Address := 16#AA18#
  --Data_reg_addr : constant Address := 16#AA1A#
  CH4csr : device_register_types.csr;
  --for CH4csr'address use Control_reg_addr;
  -- define the data register 
  CH4dbr : ch4_reading;
  --for CH4dbraddress use Data_reg_addr;
  JITTER_RANGE : constant ch4_reading := 40;
  now : TIme;

  --procedure PERIODIC_CODE is separate;
  --procedure initialise is separate;

procedure initialise is
begin
  -- enable device
  CH4csr.device := D_enabled;
end initialise;

procedure PERIODIC_CODE is

begin
    CH4csr.operation := set; -- start conversion
    now := clock;   -- wait for conversion
    loop
      exit when clock >= now + Milliseconds(100);
    end loop;
    if not CH4csr.done then
      operator_console.alarm(CH4_DEVICE_ERROR);

    else
      -- read device register for sensor value
      CH4_present := CH4dbr;
      if CH4_present > CH4_HIGH then
        if CH4_status.read = motor_safe then 
          motor.not_safe; 
          CH4_status.write(motor_unsafe);
          operator_console.alarm(high_methane);
        end if;
      elsif (CH4_present < (CH4_HIGH - jitter_range)) and 
            (CH4_status.read = motor_unsafe) then 
        motor.is_safe; 
        CH4_status.write(motor_safe);
      end if;
      data_logger.CH4_log(CH4_present);
    end if;
end PERIODIC_CODE;

  task THREAD is
    PRAGMA PRIORITY(ch4_sensor_rtatt.
                   thread_priority);
  end thread;

  task body THREAD is
    T: TIME;
    PERIOD : Time_span := ch4_sensor_rtatt.period;
  begin
    T:= clock;
    initialise;
    loop
      PERIODIC_CODE;
      T := T + PERIOD;
      delay until(T);
    end loop;
  end THREAD;
end CH4_sensor;
