with device_register_types; use device_register_types;
with system; use system;
with motor; use motor;
with data_logger;
package body HLW_handler is

  -- define control and status registers
  -- for the high and low water switches
  --HW_Control_reg_addr : constant Address := 16#AA10#
  --LW_Control_reg_addr : constant Address := 16#AA12#
  hwcsr : device_register_types.csr;
  --for hwcsraddress use HW_Control_reg_addr;
  lwcsr : device_register_types.csr;
  --for lwcsr'address use LW_Control_reg_addr;
  --procedure initialise is separate;
  --procedure sporadic_code(int : water_mark) is separate;

procedure sporadic_code(int : water_mark) is
begin
  if int = high then
    motor.set_pump(on);
    data_logger.high_low_water_log(high);
  else
    motor.set_pump(off);
    data_logger.high_low_water_log(low);
  end if;
end sporadic_code;

procedure initialise is
begin
  hwcsr.device :=D_enabled;
  hwcsr.interrupt := I_enabled;
  lwcsr.device :=D_enabled;
  lwcsr.interrupt := I_enabled;
end initialise;

  task thread is
    pragma PRIORITY(HLW_handler_rtatt.
                    thread_priority);
  end thread;

  protected Agent is
    pragma PRIORITY(HLW_handler_rtatt.
                  ceiling_priority);
    -- for the Start operation
    procedure START(int : water_mark);
    entry WAIT_START(int : out water_mark);
  private
    START_OPEN : BOOLEAN := FALSE;
    W : water_mark;
  end Agent;

  procedure start(int : water_mark) is
  begin
     Agent.start(int);
  end start;

  protected body Agent is
    procedure START(int : water_mark) is
    begin
      W := int;
      START_OPEN := TRUE;

    end START;

    entry WAIT_START(int : out water_mark) 
            when START_OPEN is
    begin
      int := W;
      START_OPEN := FALSE;
    end WAIT_START;
  end Agent;

  task body thread is
    int : water_mark; 
  begin
    initialise;
    loop
      Agent.WAIT_START(int);
      sporadic_code(int);
    end loop;
  end THREAD;
end HLW_handler;
