with data_logger;
with CH4_status; use  CH4_status;
with device_register_types; use device_register_types;
with system; use system;
with motor_rtatt; use motor_rtatt;
package body motor is

    -- define control and status register
    -- for the motor
  --Control_reg_addr : constant Address := 16#AA14#;
  pcsr : device_register_types.csr :=
      (error_bit => clear, operation => set,
       done => false, interrupt => I_enabled,
       device => D_enabled);
  --for pcsr'address use Control_reg_addr;

  protected Agent is
    PRAGMA PRIORITY(motor_rtatt.
                    ceiling_priority);
    procedure not_safe;
    procedure is_safe;
    function request_status return pump_status;
    procedure set_pump(to : pump_status);
  private
    motor_status : pump_status := off;
    motor_condition : pump_condition := enabled;
  end Agent;

  procedure not_safe is
  begin
    Agent.not_safe; 
  end not_safe;

  procedure is_safe is
  begin
    Agent.is_safe; 
  end is_safe;

  function request_status return pump_status is
  begin
      return Agent.request_status; 
  end request_status;

  procedure set_pump(to : pump_status) is
  begin
      Agent.set_pump(to);
  end set_pump;


  protected body Agent is
    procedure not_safe is
    begin
      if motor_status = on then
        pcsr.operation := clear; -- turn off motor
      end if;
      motor_condition := disabled;
      data_logger.motor_log(motor_unsafe);
    end not_safe;

    procedure is_safe is
    begin
      if motor_status = on then
        pcsr.operation := set; -- start motor
        data_logger.motor_log(motor_started);
        motor_condition := enabled;
      end if;
      data_logger.motor_log(motor_safe);
    end is_safe;


    function request_status return pump_status is
    begin
      return motor_status;
    end request_status;


    procedure set_pump(to : pump_status) is
    begin
      if to = on then 
        if motor_status = off or 
           motor_condition = disabled then
          if CH4_status.read  = motor_safe then 
            -- Environment_Monitor.Check_Safe
            motor_status := on;
            motor_condition := enabled; 
            pcsr.operation := set; -- turn on motor
            data_logger.motor_log(motor_started);
          elsif motor_condition = disabled then
            raise pump_NOT_SAFE;
          end if;
        end if;
      else
        if motor_status = on then 
          pcsr.operation := clear; -- turn off motor
          motor_status:= off;
          data_logger.motor_log(motor_stopped);
        end if;
      end if;
    end set_pump;
  end Agent;
end motor;
